function stats = algorithm_hetMan_2reg(sector_param,factor_param,...
                    solver_param)

% Title: algorithm_hetMan_2reg.m
% Date: 3/30/2013
% Author: Kevin Lim
% Description: 
%   This file implements the bvp4c boundary value problem solver for the 
%   file masterSolver_hetMan_2reg.m. Inputs are sector-specific parameters
%   (sector_param), factor-specific parameters (factor_param), and solver
%   parameters (solver_param). Statistics of interest are stored as output
%   in the cell array stats.
      

%% SECTION 1: extract parameters

% extract sector-specific parameters and functions
gamma_1 = sector_param{1};
alpha_1 = sector_param{2};
beta_1 = sector_param{3};
rho_1 = sector_param{4};
p_1 = sector_param{5};
psi_1 = sector_param{6};
gamma_2 = sector_param{7};
alpha_2 = sector_param{8};
beta_2 = sector_param{9};
rho_2 = sector_param{10};
p_2 = sector_param{11};
psi_2 = sector_param{12};

% extract factor-specific parameters and functions
qL_min = factor_param{1};
qL_max = factor_param{2};
qH_min = factor_param{3};
qH_max = factor_param{4};
L = factor_param{5};
H = factor_param{6};
k_L = factor_param{7};
k_H = factor_param{8};
phi_L = factor_param{9};
phi_H = factor_param{10};

% extract solver parameters
eq_type = solver_param{1};
init_wage = solver_param{2};
qL_gridSize = solver_param{3};
profit_tol = solver_param{4};
max_iter = solver_param{5};
bvp4c_opt = solver_param{6};


%% SECTION 2: Compute equilibrium

% Initialise algorithm success indicator and equilibrium type
foundEq = 0;
foundEq_type = 0;

for ieq = 1:length(eq_type)
    
    % set equilibrium type being considered
    eq = eq_type(ieq);
       
    % initialize variables
    niter = 1;                  % iteration counter
    qL_cutoff_max = qL_max;     % upper bound for qL_cutoff guess
    qL_cutoff_min = qL_min;     % lower bound for qL_cutoff guess
        
    while foundEq == 0 && niter <= max_iter
        
        if eq == 1  % single-sector equilibrium

            % command window messages
            if niter == 1
                disp('Looking for single-sector equilibrium:');
            end

            % set qL cutoff to qL_max
            qL_cutoff = qL_max;

            % set initial guess for bvp4c
            xinit = linspace(qL_min,qL_max,qL_gridSize);
            solinit = bvpinit(xinit,@yinit_1sec);

            % solve bvp and catch solver breakdown error
            try
                sol = bvp4c(@odefun_1sec,@bcfun_1sec,solinit,bvp4c_opt);
                qL_grid = sol.x';
                wage = sol.y(1,:)';
                match = sol.y(2,:)';
            catch err
                if(strcmp(err.identifier,'MATLAB:bvp4c:SingJac'))
                    disp('   Equilibrium not found.');
                    disp(' ');
                    break;
                else
                    rethrow(err);
                end
            end

            % Compute equilibrium profit function
            profit = computeProfit(gamma_1,p_1,psi_1,qL_grid,wage,match);

            % Compute shadow profits of managers in sector 1 if they
            % switched to sector 2
            [m_inv_high,m_inv_high_ind] = computeShadowMatch(gamma_2,...
                                            psi_2,qL_grid,wage,match);
            shadow_profit_1 = computeProfit(gamma_2,p_2,psi_2,...
                                m_inv_high,wage(m_inv_high_ind),match);


            % check if solution is consistent with equilibrium
            check_1 = min(profit - shadow_profit_1) > -profit_tol;
            if check_1 == 1
                foundEq = 1;
            else
                niter = max_iter;
                disp('   Equilibrium not found.');
                disp(' ');
            end

        elseif eq == 2 || eq == 3 % two-sector equilibrium

            % command window messages                    
            if niter == 1
                if eq == 2
                    disp('Looking for HH/LL equilibrium:');
                elseif eq == 3
                    disp('Looking for HL/LH equilibrium:');
                end
            end

            % set current guess of qL cutoff
            qL_cutoff = (qL_cutoff_min+qL_cutoff_max)/2;

            % set initial guess for bvp4c
            xinit = [linspace(qL_min,qL_cutoff,floor(qL_gridSize/2)),...
                     linspace(qL_cutoff,qL_max,ceil(qL_gridSize/2))];
            if eq == 2
                solinit = bvpinit(xinit,@yinit_2sec_HHLL);
            elseif eq == 3
                solinit = bvpinit(xinit,@yinit_2sec_HLLH);
            end

            % solve bvp and catch solver breakdown error
            try
                if eq == 2
                    sol = bvp4c(@odefun_2sec,@bcfun_2sec_HHLL,solinit,...
                            bvp4c_opt);
                elseif eq == 3
                    sol = bvp4c(@odefun_2sec,@bcfun_2sec_HLLH,solinit,...
                            bvp4c_opt);
                end
                qL_grid = sol.x';
                wage = sol.y(1,:)';
                match = sol.y(2,:)';
            catch err
                if(strcmp(err.identifier,'MATLAB:bvp4c:SingJac'))
                    disp('   Equilibrium not found.');
                    disp(' ');
                    break;
                else
                    rethrow(err);
                end
            end

            % Compute equilibrium profit function
            profit_1 = computeProfit(gamma_1,p_1,psi_1,...
                                        qL_grid,wage,match);
            profit_2 = computeProfit(gamma_2,p_2,psi_2,...
                                        qL_grid,wage,match);
            profit = [profit_2(1:find(qL_grid==qL_cutoff,1));
                      profit_1(find(qL_grid==qL_cutoff,1)+1:end)];

            % Compute shadow profits of managers in sector 1 if they
            % switched to sector 2
            [m_inv_high,m_inv_high_ind] = computeShadowMatch(gamma_2,...
                                            psi_2,qL_grid,wage,match);
            shadow_profit_1 = computeProfit(gamma_2,p_2,psi_2,...
                                m_inv_high,wage(m_inv_high_ind),match);

            % Compute shadow profits of managers in sector 2 if they
            % switched to sector 1
            [m_inv_high,m_inv_high_ind] = computeShadowMatch(gamma_1,...
                                            psi_1,qL_grid,wage,match);
            shadow_profit_2 = computeProfit(gamma_1,p_1,psi_1,...
                                m_inv_high,wage(m_inv_high_ind),match);

            % Check if solution is consistent with equilibrium:
            %  - to determine the direction of adjustment for the 
            %    qL cutoff search, the algorithm omits checks for
            %    profitable deviations within the same sector
            %  - however, the final check for consistency with
            %    equilibrium is over the whole qL space

            % checks for profitable deviations over entire qL space
            check_1 = min(profit-shadow_profit_1) > -profit_tol;
            check_2 = min(profit-shadow_profit_2) > -profit_tol;

            % checks for profitable deviations from one sector to
            % another, including qL_cutoff
            check_1_adj = min(profit(qL_grid >= qL_cutoff) -...
                            shadow_profit_1(qL_grid >= qL_cutoff))...
                            > -profit_tol;
            check_2_adj = min(profit(qL_grid <= qL_cutoff) -...
                            shadow_profit_2(qL_grid <= qL_cutoff))...
                            > -profit_tol;

            % checks for profitable deviations from one sector to
            % another, excluding qL_cutoff
            check_1_adj_ex = min(profit(qL_grid > qL_cutoff) -...
                            shadow_profit_1(qL_grid > qL_cutoff))...
                            > -profit_tol;
            check_2_adj_ex = min(profit(qL_grid < qL_cutoff) -...
                            shadow_profit_2(qL_grid < qL_cutoff))...
                            > -profit_tol;


            % no profitable deviations found            
            if check_1 == 1 && check_2 == 1
                % so equilibrium found
                foundEq = 1;       
            % profitable deviations exist from high to low sector,
            % but not from low to high sector
            elseif check_1_adj == 0 && check_2_adj == 1
                % so adjust qL cutoff upwards
                qL_cutoff_min = qL_cutoff;
            % profitable deviations exist from low to high sector,
            % but not from high to low sector                        
            elseif check_1_adj == 1 && check_2_adj == 0
                % so adjust qL cutoff downwards
                qL_cutoff_max = qL_cutoff;
            else
                if check_1_adj_ex == 0 && check_2_adj_ex == 1
                    qL_cutoff_min = qL_cutoff;
                elseif check_1_adj_ex == 1 && check_2_adj_ex == 0
                    qL_cutoff_max = qL_cutoff;
                else
                    disp('   Equilibrium not found.');
                    disp(' ');
                    niter = max_iter;      
                end
            end

        end

        % display qL cutoff if looking for two-sector equilibrium
        if eq~=1 && (niter ~= max_iter || foundEq == 1)
            display(['   qL_cutoff = ',num2str(qL_cutoff)])
        end

        % increment iteration counter
        niter = niter + 1;


    end

    % set equilibrium type found and number of regions, and display success
    % message
    if foundEq
        foundEq_type = eq;
        disp('   Equilibrium found.');
        disp(' ');
        break;
    end

    
    if foundEq, break; end
end

% Compute output levels 
if foundEq
    if foundEq_type == 1
        output_1 = computeOutput(gamma_1,p_1,qL_grid,wage);
        output_2 = 0;
    elseif foundEq_type == 2 || foundEq_type == 3
        index_1 = qL_grid>=qL_cutoff;
        index_2 = qL_grid<=qL_cutoff;
        output_1 = computeOutput(gamma_1,p_1,qL_grid(index_1),...
                    wage(index_1));
        output_2 = computeOutput(gamma_2,p_2,qL_grid(index_2),...
                    wage(index_2));
    end
else
    output_1 = 0;
    output_2 = 0;
      
end

% compute qH cutoff
switch foundEq_type
    case 0
        qH_cutoff = [];
    case 1
        qH_cutoff = qH_max;
    case 2
        qH_cutoff = match(find(qL_grid==qL_cutoff,1));
    case 3
        qH_cutoff = match(1);
end

% display message if no equilibrium found
if ~foundEq, display('No equilibrium found.'); end

%% SECTION 3: Store results
stats = {foundEq,foundEq_type,qL_grid,wage,match,profit,...
            qL_cutoff,qH_cutoff,output_1,output_2};

%% SECTION 4: Define functions used by algorithm

% ode functions for bvp4c

% ode for single-sector equilibrium
function dy = odefun_1sec(qL,y)
            dw = y(1)/qL/gamma_1*(alpha_1*qL^rho_1*(alpha_1+beta_1))...
                    /(alpha_1*qL^rho_1+beta_1*y(2)^rho_1);
            dm = (L/H)*(phi_L(qL)/phi_H(y(2)))...
                    *(y(1)/(gamma_1*p_1*psi_1(y(2),qL)))^(1/(1-gamma_1));
            dy = [dw;
                  dm];
end

% ode for two-sector two-region equilibrium
function dy = odefun_2sec(qL,y,region)
    switch region
        case 1
            dw = y(1)/qL/gamma_2*(alpha_2*qL^rho_2*(alpha_2+beta_2))...
                    /(alpha_2*qL^rho_2+beta_2*y(2)^rho_2);
            dm = (L/H)*(phi_L(qL)/phi_H(y(2)))...
                    *(y(1)/(gamma_2*p_2*psi_2(y(2),qL)))^(1/(1-gamma_2));
            dy = [dw;
                  dm];
        case 2
            dw = y(1)/qL/gamma_1*(alpha_1*qL^rho_1*(alpha_1+beta_1))...
                    /(alpha_1*qL^rho_1+beta_1*y(2)^rho_1);
            dm = (L/H)*(phi_L(qL)/phi_H(y(2)))...
                    *(y(1)/(gamma_1*p_1*psi_1(y(2),qL)))^(1/(1-gamma_1));
            dy = [dw;
                  dm];
    end
end

% boundary conditions for bvp4c

% boundary conditions for one-sector equilibrium
function res = bcfun_1sec(yl,yr)
    res = [yl(2) - qH_min
           yr(2) - qH_max];
end

% boundary conditions for two-sector HH/LL equilibrium
function res = bcfun_2sec_HHLL(yl,yr)
    res = [yl(2,1) - qH_min
           yr(2,2) - qH_max
           yr(1,1) - yl(1,2)
           yr(2,1) - yl(2,2)];
end

% boundary conditions for two-sector HL/LH equilibrium
function res = bcfun_2sec_HLLH(yl,yr)
    res = [yr(2,1) - qH_max
           yl(2,2) - qH_min
           yl(2,1) - yr(2,2)
           yr(1,1) - yl(1,2)];
end

% initial guess for bvp4c

% initial guess for one-sector equilibrium
function y = yinit_1sec(qL)
    wrange_min = init_wage;
    wrange_max = wrange_min + (qL_max-qL_min);
    y = [wrange_min+(wrange_max-wrange_min)/(qL_max-qL_min)*(qL-qL_min);
             qH_min + (qH_max-qH_min)/(qL_cutoff-qL_min)*(qL-qL_min)];
end

% initial guess for two-sector HH/LL equilibrium
function y = yinit_2sec_HHLL(qL,region)
    wrange_min = init_wage;
    wrange_max = wrange_min + (qL_max-qL_min);
    wrange_mid = (wrange_max+wrange_min)/2;
    qH_mid = qH_min + (qH_max-qH_min)/2;
    switch region
        case 1
            y = [wrange_min+(wrange_mid-wrange_min)...
                    /(qL_cutoff-qL_min)*(qL-qL_min);
                 qH_min + (qH_mid-qH_min)/(qL_cutoff-qL_min)*(qL-qL_min)];
        case 2
            y = [wrange_mid+(wrange_max-wrange_mid)...
                    /(qL_max-qL_cutoff)*(qL-qL_cutoff);
                 qH_mid + (qH_max-qH_mid)/(qL_max-qL_cutoff)...
                    *(qL-qL_cutoff)];
    end
end

% initial guess for two-sector HL/LH equilibrium
function y = yinit_2sec_HLLH(qL,region)
    wrange_min = init_wage;
    wrange_max = wrange_min + (qL_max-qL_min);
    wrange_mid = (wrange_max+wrange_min)/2;
    qH_mid = qH_min + (qH_max-qH_min)/2;
    switch region
        case 1
            y = [wrange_min+(wrange_mid-wrange_min)...
                /(qL_cutoff-qL_min)*(qL-qL_min);
                 qH_mid + (qH_max-qH_mid)/(qL_cutoff-qL_min)...
                    *(qL-qL_min)];
        case 2
            y = [wrange_mid+(wrange_max-wrange_mid)...
                    /(qL_max-qL_cutoff)*(qL-qL_cutoff);
                 qH_min + (qH_mid-qH_min)/(qL_max-qL_cutoff)...
                    *(qL-qL_cutoff)];
    end
end

% profit, shadow match, and output functions

% profit function
function profit = computeProfit(gamma,p,psi,qL_grid,w,m)
    gammaBar = gamma^(gamma/(1-gamma))-gamma^(1/(1-gamma));
    profit = gammaBar*p^(1/(1-gamma))*(psi(m,qL_grid)).^(1/(1-gamma))...
                .*(w.^(-gamma/(1-gamma)));
end

% shadow match function
function [m_inv_shadow,m_inv_shadow_ind] = ...
    computeShadowMatch(gamma_shadow,psi_shadow,qL_grid,w,m)

    % Initialize inverse match function in shadow sector
    m_inv_shadow = nan(length(qL_grid),1);
    m_inv_shadow_ind = nan(length(qL_grid),1);

    % Iterate over m(qL_grid) values, finding optimal match for each
    for i=1:length(qL_grid)
        [~,optimal_qL] = max(psi_shadow(m(i),qL_grid).*w.^(-gamma_shadow));
        m_inv_shadow_ind(i) = optimal_qL;
        m_inv_shadow(i) = qL_grid(optimal_qL);

    end
end

% output function
function output = computeOutput(gamma,p,qL_grid,w)
    output = (L/(gamma*p))*trapz(qL_grid,...
                        w.*phi_L(qL_grid),1);
end

end