% Title: masterSolver_hetMan_2reg.m
% Date: 3/30/2013
% Author: Kevin Lim
% Description: 
%   This file solves a special case of the GHK model in which:
%       (1) managers are heterogenous in ability
%       (2) the productivity of firms in sector i that hire workers of 
%           ability qL and managers of ability qH is
%           psi_i(qH,qL) = (alpha_i*qL^rho_i+beta_i*qH^rho_i)...
%                           ^((alpha_i+beta_i)/rho_i)
%       (3) the equilibrium sorting pattern is characterized by two regions
%           of sorting
%   Note that the algorithm adheres to the convention that the best workers 
%   always sort to sector 1, relabeling sectors if necessary.


%% SECTION 1: Set baseline economic parameters and solver settings

clear all; close all; clc;

% Set sector-specific parameters and functions
gamma_1 = .3;       % prod fxn elasticity wrt worker quantity, sector 1
gamma_2 = .2;       % prod fxn elasticity wrt worker quantity, sector 2
alpha_1 = 1;       % as in description of file
alpha_2 = .5;       % as in description of file
beta_1 = 2-alpha_1; % as in description of file
beta_2 = 1-alpha_2; % as in description of file
rho_1 = -5;        % as in description of file
rho_2 = -5;        % as in description of file
p_1 = 1;            % goods price, sector 1
p_2 = 1;            % goods price, sector 2
psi_1 = @(qH,qL) (alpha_1*qL.^rho_1 + beta_1*qH.^rho_1)...
        .^((alpha_1+beta_1)/rho_1);     % as in description of file
psi_2 = @(qH,qL) (alpha_2*qL.^rho_2 + beta_2*qH.^rho_2)...
        .^((alpha_2+beta_2)/rho_2);     % as in description of file

% concatenate sector-specific parameters
sector_param_1 = {gamma_1,alpha_1,beta_1,rho_1,p_1,psi_1};
sector_param_2 = {gamma_2,alpha_2,beta_2,rho_2,p_2,psi_2};
sector_param = [sector_param_1,sector_param_2];

% Set factor-specific parameters and functions
qL_min = 1;     % worker ability distribution, lower bound
qL_max = 2;     % worker ability distribution, upper bound
qH_min = 1;     % manager ability distribution, lower bound
qH_max = 2;     % manager ability distribution, upper bound
L = 1;          % aggregate worker endowment
H = 1;          % aggregate manager endowment
k_L = 3;        % shape parameter of truncated Pareto distribution 
                % of worker abilities
k_H = 3;        % shape parameter of truncated Pareto distribution 
                % of manager abilities
phi_L = @(qL) k_L*qL_min^k_L*qL.^(-k_L-1)...
        /(1-(qL_min/qL_max)^k_L);    % density function of worker ability
phi_H = @(qH) k_H*qH_min^k_H*qH.^(-k_H-1)...
        /(1-(qH_min/qH_max)^k_H);    % density function of manager ability

% concatenate factor-specific parameters
factor_param = {qL_min,qL_max,qH_min,qH_max,L,H,k_L,k_H,phi_L,phi_H};

% Set solver parameters
eq_type = [1 2 3];  % order of equilibrium types to look for
                    %  - 1: single-sector
                    %  - 2: two-sector HH/LL
                    %  - 3: two-sector HL/LH
init_wage = .01;     % wage at qL=qL_min for initial guess of wage solution
qL_gridSize = 200;  % (approximate) number of gridpoints between
                    % qL_min and qL_max
profit_tol = 1E-5;  % bandwidth acceptable for profit differentials
max_iter = 100;     % maximum number of iterations
bvp4c_opt = bvpset('AbsTol',1e-6,'RelTol',1e-6); % options for bvp4c

% concatenate solver parameters
solver_param = {eq_type,init_wage,qL_gridSize,profit_tol,max_iter,...
                bvp4c_opt};

% initialise indicators
relabel = 0;    % indiactor = 1 if sectors have been relabeled
foundEq = 0;    % indicator = 1 if equilibrium successfully found


%% Section 2: check sufficient conditions for sorting (equal rho only)
if rho_1 == rho_2
   
    rho = rho_1;
    r_1 = beta_1/alpha_1;
    r_2 = beta_2/alpha_2;   
    s_1 = gamma_1/gamma_2*(alpha_2+beta_2)/(alpha_1+beta_1);
    s_2 = (1-gamma_1)/(1-gamma_2)*r_2/r_1*(alpha_2+beta_2)...
            /(alpha_1+beta_1);
    
    % HH/LL sorting
    if r_2 > r_1
        c_1 = (1+r_2*(qH_max/qL_min)^rho)/(1+r_1*(qH_max/qL_min)^rho);
        c_2 = (1+r_2*(qH_min/qL_max)^rho)/(1+r_1*(qH_min/qL_max)^rho);
    else
        c_1 = (1+r_2*(qH_min/qL_max)^rho)/(1+r_1*(qH_min/qL_max)^rho);
        c_2 = (1+r_2*(qH_max/qL_min)^rho)/(1+r_1*(qH_max/qL_min)^rho);
    end
    if c_1 > max(s_1,s_2)
         disp('Sufficient conditions met for HH/LL equilibrium.')
         disp(' ');
    end
    if c_2 < min(s_1,s_2)
         disp(['Sufficient conditions met for HH/LL equilibrium'...
                ' (sectors reversed).']);
         disp(' ');
    end

    % HL/LH sorting
    if r_2/r_1 < (qH_min/qH_max)^rho
        c_1 = (qL_max^rho+r_2*qH_max^rho)/(qL_max^rho+r_1*qH_min^rho);
    else
        c_1 = (qL_min^rho+r_2*qH_max^rho)/(qL_min^rho+r_1*qH_min^rho);
    end
    if r_2/r_1 > (qL_min/qL_max)^rho
        c_2 = (qL_min^rho+r_2*qH_min^rho)/(qL_max^rho+r_1*qH_min^rho);
    else
        c_2 = (qL_min^rho+r_2*qH_max^rho)/(qL_max^rho+r_1*qH_max^rho);
    end
    if r_2/r_1 < (qH_max/qH_min)^rho
        c_3 = (qL_min^rho+r_2*qH_min^rho)/(qL_min^rho+r_1*qH_max^rho);
    else
        c_3 = (qL_max^rho+r_2*qH_min^rho)/(qL_max^rho+r_1*qH_max^rho);
    end
    if r_2/r_1 > (qL_max/qL_min)^rho
        c_4 = (qL_max^rho+r_2*qH_max^rho)/(qL_min^rho+r_1*qH_max^rho);
    else
        c_4 = (qL_max^rho+r_2*qH_min^rho)/(qL_min^rho+r_1*qH_min^rho);
    end
    if  c_1 > s_1 && c_2 < s_2
         disp('Sufficient conditions met for HL/LH equilibrium.')
         disp(' ');
    end
    if c_3 < s_1 && c_4 > s_2 
         disp(['Sufficient conditions met for HL/LH equilibrium'...
                ' (sectors reversed).']);
         disp(' ');
    end
end


%% SECTION 3: Solve for equilibrium

% execute algorithm if equilibrium not yet found and sectors have not
% been relabeled
while foundEq == 0 && relabel ~= 2

    % relabel sectors if necessary
    if relabel == 1

        disp(' ');
        disp('Relabeling sectors...');
        disp(' ');

        nsp = length(sector_param)/2;
        sector_param = [sector_param(nsp+1:end), sector_param(1:nsp)];

    end

    % run algorithm
    stats = algorithm_hetMan_2reg(sector_param,factor_param,solver_param);

    % extract results
    foundEq = stats{1};         % indicator = 1 if equilibrium successfully
                                % found
    foundEq_type = stats{2};    % type of equilibrium found
                                %  - 1: single-sector
                                %  - 2: two-sector HH/LL
                                %  - 3: two-sector HL/LH
    qL_grid = stats{3};         % qL grid
    wage = stats{4};            % wage function
    match = stats{5};           % matching function
    profit = stats{6};          % profit function
    qL_cutoff = stats{7};       % qL cutoff value
    qH_cutoff = stats{8};       % qH cutoff value
    output_1 = stats{9};        % sector 1 output
    output_2 = stats{10};       % sector 2 output
    
    % if equilibrium not found, relabel sectors
    if foundEq == 0
        relabel = relabel + 1;
    end

end

%% SECTION 4: Plot and display results

% if equilibrium found, plot and display results
if foundEq == 1
    
    % display type of equilibrium found
    switch foundEq_type
        case 1, nregions = 1; disp('Single-sector equilibrium found!');
        case 2, nregions = 2; disp('HH/LL equilibrium found!');
        case 3, nregions = 2; disp('HL/LH equilibrium found!');
    end
    
    % get qL cutoff indices
    if foundEq_type == 1
        qL_cutoff_index = length(qL_grid);
    elseif foundEq_type == 2 || foundEq_type == 3
        qL_cutoff_index = find(qL_grid==qL_cutoff,1);
    end

    % initialize figure
    figure('units','normalized','outerposition',[0 0 1 1])
    fsize = 20;

    % Plot matching function
    plot_match = subplot(2,2,1); hold on;
    if foundEq == 1
        if foundEq_type == 1
            
            plot(qL_grid,match,'Color','b','LineWidth',.5);
            leg_match = legend('Sector 1');
            
        elseif foundEq_type == 2 || foundEq_type == 3

            plot(qL_grid(qL_cutoff_index+1:end),...
                    match(qL_cutoff_index+1:end),...
                    'Color','b','LineWidth',.5);
            plot(qL_grid(1:qL_cutoff_index),...
                    match(1:qL_cutoff_index),...
                    'Color','r','LineWidth',4);
            leg_match = legend('Sector 1','Sector 2');
            
        end

        if foundEq_type == 2
            set(leg_match,'Location','SouthEast'); 
        elseif foundEq_type == 3
            set(leg_match,'Location','NorthEast'); 
        end

    end
    hold off;
    title('Matching function','FontSize',fsize);
    xlabel('q_L','FontSize',fsize);
    ylabel('q_H','FontSize',fsize);
    set(gca,'FontSize',fsize);
    axis([get(gca,'Xlim') get(gca,'Ylim')]);

    % Plot worker wage function
    plot_wage = subplot(2,2,2); hold on;
    if foundEq == 1
        if foundEq_type == 1
            
            plot(qL_grid,wage,'Color','b','LineWidth',.5);
            leg_wage = legend('Sector 1');
            
        elseif foundEq_type == 2 || foundEq_type == 3

            plot(qL_grid(qL_cutoff_index+1:end),...
                    wage(qL_cutoff_index+1:end),...
                    'Color','b','LineWidth',.5);
            plot(qL_grid(1:qL_cutoff_index),...
                    wage(1:qL_cutoff_index),...
                    'Color','r','LineWidth',4);
            leg_wage = legend('Sector 1','Sector 2');
            
        end

        set(leg_wage,'Location','SouthEast'); 

    end
    hold off;
    title('Worker wage function','FontSize',fsize);
    xlabel('q_L','FontSize',fsize);
    ylabel('w(q_L)','FontSize',fsize);
    set(gca,'FontSize',fsize);

    % Plot manager salary function
    plot_salary = subplot(2,2,3); hold on;
    if foundEq == 1
        if foundEq_type == 1

            plot(qL_grid,profit,'Color','b','LineWidth',.5);
            leg_profit = legend('Sector 1');

        elseif foundEq_type == 2 || foundEq_type == 3

            plot(match(qL_cutoff_index+1:end),...
                    profit(qL_cutoff_index+1:end),...
                    'Color','b','LineWidth',.5);
            plot(match(1:qL_cutoff_index),...
                    profit(1:qL_cutoff_index),...
                    'Color','r','LineWidth',4);
            leg_profit = legend('Sector 1','Sector 2');

        end

        set(leg_profit,'Location','SouthEast'); 

    end
    hold off;
    title('Manager salary function','FontSize',fsize);
    xlabel('q_H','FontSize',fsize);
    ylabel('r(q_H)','FontSize',fsize);
    set(gca,'FontSize',fsize);

    % display qL and qH cutoff values
    disp(['   qL cutoff value = ' num2str(qL_cutoff)]);
    disp(['   qH cutoff value = ' num2str(qH_cutoff)]);
    
    % display output levels
  	disp(['   Sector 1 output = ' num2str(output_1)]);
  	disp(['   Sector 2 output = ' num2str(output_2)]);
end
