% Title: masterSolver_homMan_3reg.m
% Date: 3/30/2013
% Author: Kevin Lim
% Description: 
%   This file solves a special case of the GHK model in which:
%       (1) managers are homogenous in ability
%       (2) the productivity of firms in sector i that hire workers of 
%           ability qL is psi_i(qL) = (alpha_i*qL^rho_i+1)^(1/rho_i)
%       (3) the equilibrium sorting pattern is characterized by three 
%           regions of sorting
%   Note that the algorithm adheres to the convention that the best workers 
%   always sort to sector 1.


%% SECTION 1: Set baseline economic parameters and solver settings

clear all; close all; clc;

% Set sector-specific parameters
gamma_1 = .5;   % prod fxn elasticity wrt worker quantity, sector 1
gamma_2 = .5;   % prod fxn elasticity wrt worker quantity, sector 2
gamma_bar_1 = gamma_1^(gamma_1/(1-gamma_1))*(1-gamma_1);
gamma_bar_2 = gamma_2^(gamma_2/(1-gamma_2))*(1-gamma_2);
alpha_1 = .2;   % as in description of file
alpha_2 = .8;   % as in description of file
rho_1 = -1;     % as in description of file
rho_2 = -20;    % as in description of file
p_1 = 1;        % goods price, sector 1
p_2 = .9;       % goods price, sector 2

% Set factor-specific parameters and functions
qL_min = .5;    % worker ability distribution, lower bound
qL_max = 2.5;   % worker ability distribution, upper bound
L = 1;          % aggregate worker endowment
H = 1;          % aggregate manager endowment
k_L = 3;        % shape parameter of truncated Pareto distribution 
                % of worker abilities
phi_L = @(qL) k_L*qL_min^k_L*qL.^(-k_L-1)...
        /(1-(qL_min/qL_max)^k_L);   % density function of worker ability

% initialise indicators
relabel = 0;    % indiactor = 1 if sectors have been relabeled
foundEq = 0;    % indicator = 1 if equilibrium successfully found

% set tolerance and max iterations
ld_tol = 5E-5;      % tolerance for excess effective labor supply
qL_gridSize = 200;  % qL grid size
max_iter = 100;     % maximum number of iterations

%% SECTION 2: Solve for equilibrium

% execute algorithm if equilibrium not yet found and sectors have not been
% relabeled
while foundEq == 0 && relabel ~= 2
    
    % initialise variables
    niter = 1;      % iteration counter
    w_1_min = 0;    % initial guess for sector 1 wage anchor, lower bound
    w_1_max = 100;  % initial guess for sector 1 wage anchor, upper bound
    LS = Inf;       % excess effective labor supply

    % relabel sectors if necessary
    if relabel == 1
        
        disp(' ');
        disp('Relabeling sectors...');
        disp(' ');
        
        gamma_old = [gamma_1,gamma_2];
        gamma_1 = gamma_old(2);
        gamma_2 = gamma_old(1);
    
        gamma_bar_old = [gamma_bar_1,gamma_bar_2];
        gamma_bar_1 = gamma_bar_old(2);
        gamma_bar_2 = gamma_bar_old(1);

        alpha_old = [alpha_1,alpha_2];
        alpha_1 = alpha_old(2);
        alpha_2 = alpha_old(1);

        rho_old = [rho_1,rho_2];
        rho_1 = rho_old(2);
        rho_2 = rho_old(1);
        
        p_old = [p_1,p_2];
        p_1 = p_old(2);
        p_2 = p_old(1);
        
    end
    
    % iterate if excess effective labor supply tolerance not met and 
    % iteration count less than maximum iteration count
    while abs(LS) > ld_tol && niter < max_iter

        % display iteration statistics
        disp(['Iteration ' num2str(niter)]);

        % set guess for sector 1 wage anchor
        w_1 = .5*(w_1_min+w_1_max);

        % solve for sector 2 wage anchor
        w_2 = (gamma_bar_2/gamma_bar_1*p_2^(1/(1-gamma_2))/...
                p_1^(1/(1-gamma_1))*w_1^(gamma_1/(1-gamma_1)))...
                ^((1-gamma_2)/gamma_2);

        % define wage functions
        wfunc_1 = @(qL) w_1 .* (alpha_1.*qL.^rho_1+1).^(1/gamma_1/rho_1);
        wfunc_2 = @(qL) w_2 .* (alpha_2.*qL.^rho_2+1).^(1/gamma_2/rho_2);

        % solve for qL cutoffs
        qL_cutoff_func = @(qL) wfunc_1(qL)-wfunc_2(qL);
        fsolveoptions = optimset('TolFun',1E-10,'Display','off');
        qL_cutoff_1 = fsolve(qL_cutoff_func,.9,fsolveoptions);
        qL_cutoff_2 = fsolve(qL_cutoff_func,qL_max*.9,fsolveoptions);

        % define functions to be intergrated for labor market clearing
        intfunc_1 = @(qL) (w_1/gamma_1/p_1)^(1/(1-gamma_1))...
                            .*(alpha_1.*qL.^rho_1+1).^(1/gamma_1/rho_1)...
                            .*phi_L(qL);
        intfunc_2 = @(qL) (w_2/gamma_2/p_2)^(1/(1-gamma_2))...
                            .*(alpha_2.*qL.^rho_2+1).^(1/gamma_2/rho_2)...
                            .*phi_L(qL);

        % define qL grids
        qL_grid_1_low = linspace(qL_min,qL_cutoff_1,qL_gridSize);
        qL_grid_1_high = linspace(qL_cutoff_2,qL_max,qL_gridSize);
        qL_grid_2 = linspace(qL_cutoff_1,qL_cutoff_2,qL_gridSize);


        % compute manager allocations and excess effective labor supply
        H_1 = L * (trapz(qL_grid_1_low,intfunc_1(qL_grid_1_low))...
                    + trapz(qL_grid_1_high,intfunc_1(qL_grid_1_high)));
        H_2 = L * trapz(qL_grid_2,intfunc_2(qL_grid_2));
        LS = H/L - H_1/L - H_2/L;

        % increment iteration counter and display statistics
        niter = niter + 1;
        disp(['   excess effective labor supply = ' num2str(LS)]);
        disp(['   w_1 guess = ' num2str(w_1)]);
        disp(' ');

        % adjust guess for sector 1 wage anchor
        
        if LS > ld_tol
            
            % positive excess effective labor supply, so adjust guess 
            % upwards
            w_1_min = w_1;      
            
        elseif LS < -ld_tol
            
            % negative excess effective labor supply, so adjust guess 
            % downwards
            w_1_max = w_1;
            
        else
            
            % check that all firms earn zero profits
            qL_grid_1 = [qL_grid_1_low qL_grid_1_high];
            r = gamma_bar_1*p_1^(1/(1-gamma_1))*w_1^(-gamma_1/(1-gamma_1));
            profit_1 = gamma_bar_1*p_1^(1/(1-gamma_1))...
                        *(alpha_1.*qL_grid_1.^rho_1+1)...
                        .^(1./rho_1./(1-gamma_1)).*wfunc_1(qL_grid_1)...
                        .^(-gamma_1./(1-gamma_1))-r;
            profit_2 = gamma_bar_2*p_2^(1/(1-gamma_2))...
                        *(alpha_2.*qL_grid_2.^rho_2+1)...
                        .^(1./rho_2./(1-gamma_2)).*wfunc_2(qL_grid_2)...
                        .^(-gamma_2./(1-gamma_2))-r;
            profit = [profit_1, profit_2];

            % check for profitable deviations
            profit_1_dev = gamma_bar_1*p_1^(1/(1-gamma_1))...
                        *(alpha_1.*qL_grid_2.^rho_1+1)...
                        .^(1./rho_1./(1-gamma_1)).*wfunc_2(qL_grid_2)...
                        .^(-gamma_1./(1-gamma_1))-r;
            profit_2_dev = gamma_bar_2*p_2^(1/(1-gamma_2))...
                        *(alpha_2.*qL_grid_1.^rho_2+1)...
                        .^(1./rho_2./(1-gamma_2)).*wfunc_1(qL_grid_1)...
                        .^(-gamma_2./(1-gamma_2))-r;
            profit_dev = [profit_1_dev, profit_2_dev];
            
            if max(profit) > 1E-4
                disp('Algorithm converged but positive profits exist.');
                niter = max_iter;
            elseif max(profit_dev) > 1E-4
                disp(['Algorithm converged but'...
                        ' profitable deviations exist.']);
                niter = max_iter;
            else
                foundEq = 1;
            end
        end

    end
    
    % if equilibrium not found, relabel sectors
    if foundEq == 0
        relabel = relabel + 1;
    end
end

%% SECTION 3: Plot and display results

% if equilibrium found, plot and display results
if foundEq == 1
    
    % display success message
    disp('Equilibrium found!');
    
    % display relabel warning
    if relabel == 1
        disp(' ');
        disp('   Warning: sectors relabled.');
        disp(' ');
    end
    
    % plot wage functions
    figure('units','normalized','outerposition',[0 0 1 1]);
    hold on; fsize = 20;
    p1 = plot(qL_grid_1_low,wfunc_1(qL_grid_1_low),'Color','b',...
                'LineWidth',.5);
    plot(qL_grid_1_high,wfunc_1(qL_grid_1_high),'Color','b',...
            'LineWidth',.5);
    p2 = plot(qL_grid_2,wfunc_2(qL_grid_2),'Color','r','LineWidth',4); 
    p3 = plot(qL_grid_2,wfunc_1(qL_grid_2),'Color','b',...
            'LineStyle','--','LineWidth',.5);
    p4 = plot(qL_grid_1_low,wfunc_2(qL_grid_1_low),'Color','r',...
            'LineStyle','--','LineWidth',4);
    plot(qL_grid_1_high,wfunc_2(qL_grid_1_high),'Color','r',...
            'LineStyle','--','LineWidth',4);

    legend([p1 p2 p3 p4],'Sector 1 wages','Sector 2 wages',...
            'Sector 1 shadow wages','Sector 2 shadow wages',...
            'Location','SouthEast');
    xlabel('q_L','FontSize',fsize);
    ylabel('w(q_L)','FontSize',fsize);
    set(gca,'FontSize',fsize);
    
    % display qL cutoff value
  	disp(['   qL cutoff value 1 = ' num2str(qL_cutoff_1)]);
  	disp(['   qL cutoff value 2 = ' num2str(qL_cutoff_2)]);    
    
    % display wage anchors
  	disp(['   Sector 1 wage anchor = ' num2str(w_1)]);
  	disp(['   Sector 2 wage anchor = ' num2str(w_2)]);

    % display manager wage rate
  	disp(['   Manager salary = ' num2str(r)]);
    
    % display manager allocation
  	disp(['   Sector 1 manager allocation = ' num2str(H_1)]);
  	disp(['   Sector 2 manager allocation = ' num2str(H_2)]);

    % display sector output levels
    output_1 = H_1*(gamma_1*p_1/w_1)^(gamma_1/(1-gamma_1));
    output_2 = H_2*(gamma_2*p_2/w_2)^(gamma_2/(1-gamma_2));
  	disp(['   Sector 1 output = ' num2str(output_1)]);
  	disp(['   Sector 2 output = ' num2str(output_2)]);

end